#line 582 "./lpsrc/flx_gc.pak"
#ifndef __FLX_COLLECTOR_H__
#define __FLX_COLLECTOR_H__
#include "flx_gc.hpp"
#include "flx_gc_private.hpp"
#include <map>

namespace flx {
namespace gc {
namespace collector {
using namespace generic;

struct GC_EXTERN malloc_free;
struct GC_EXTERN flx_collector_t;

/// Allocator using malloc and free.
struct GC_EXTERN malloc_free : public virtual allocator_t
{
  void *allocate(std::size_t);
  void *reallocate(void *, std::size_t);
  void deallocate(void *);
};


/// Naive Mark and Sweep Collector.
struct GC_EXTERN flx_collector_t : public collector_t
{
  flx_collector_t(allocator_t *);
  ~flx_collector_t();

  // special to this collector ..?
  void set_min_arena_size(unsigned long);
  bool check_client_pointer(void *);
  bool check_frame_pointer(frame_t *);


protected:

  /// allocator
  void *impl_allocate(gc_shape_t *ptr_map, unsigned long);
  void impl_deallocate(frame_t *frame);

  /// collector (returns number of objects collected)
  unsigned long impl_collect();

  // add and remove roots
  void impl_add_root(void *memory);
  void impl_remove_root(void *memory);

  //
  void check();

  // statistics
  unsigned long impl_get_allocation_count()const;
  unsigned long impl_get_root_count()const;
  unsigned long impl_get_allocation_amt()const;

  void impl_compact(bool closed);
  void impl_check();

private:
  /// allocator
  void *v_allocate(gc_shape_t *ptr_map, unsigned long);
  void v_deallocate(frame_t *frame);

  /// collector (returns number of objects collected)
  unsigned long v_collect();

  // add and remove roots
  void v_add_root(void *memory);
  void v_remove_root(void *memory);

  //
  void v_check();
  // statistics
  unsigned long v_get_allocation_count()const;
  unsigned long v_get_root_count()const;
  unsigned long v_get_allocation_amt()const;

  void v_compact(bool closed);

private:
  bool collecting;
  unsigned long allocation_count;
  unsigned long root_count;
  unsigned long allocation_amt;


  void unlink(frame_t *frame);
  void dispose(frame_t *frame);
    // calls post_delete or delete_frame

  void post_delete(frame_t *frame);
  void delete_frame(frame_t *frame);
  unsigned long reap();

  void mark();
  unsigned long sweep(); // calls scan_object
  void scan_object(frame_t *frame);

  frame_t *first;
  frame_t *to_delete;
  typedef std::map<frame_t*,unsigned long, std::less<frame_t*> > rootmap_t;
  rootmap_t roots;
  bool parity;
  allocator_t *allocator;

  // if compaction is being used this stuff controls
  // the arena. The arena ptr grows DOWN towards the
  // start of the arena. arena is NULL unless arenas
  // are being used. Once in use, the allocator is
  // not (normally) used for individual objects

  void *arena;       // if compaction is being used, lo address
  void *arena_high;  // arena base (high) address
  void *arena_ptr;   // current top of the stack
  unsigned long arena_size; // hi - lo
  unsigned long arena_free; // ptr - lo
  float arena_size_factor;
  unsigned long min_arena_size;
};

}}} // end namespaces
#endif

