#line 425 "./lpsrc/flx_gc.pak"

#include "flx_gc.hpp"
#include <cstdio>
#include <cassert>
#include "flx_gc_private.hpp"

namespace flx {
namespace gc {
namespace generic {

// create a shape object given an array of offsets
gc_shape_t::gc_shape_t
(
  gc_shape_t *ns,
  char const *cn,
  std::size_t count_a,
  std::size_t amt_a,
  void (*finaliser_a)(collector_t*, void*),
  std::size_t n_offsets_a,
  std::size_t *offsets_a,
  gc_shape_flags_t flags_a
)
  :
  next_shape(ns),
  cname(cn),
  count(count_a),
  amt(amt_a),
  finaliser(finaliser_a),
  n_offsets(n_offsets_a),
  offsets(offsets_a),
  flags(flags_a)
{}

// with flags defaulted
gc_shape_t::gc_shape_t
(
  gc_shape_t *ns,
  char const *cn,
  std::size_t count_a,
  std::size_t amt_a,
  void (*finaliser_a)(collector_t*, void*),
  std::size_t n_offsets_a,
  std::size_t *offsets_a
)
  :
  next_shape(ns),
  cname(cn),
  count(count_a),
  amt(amt_a),
  finaliser(finaliser_a),
  n_offsets(n_offsets_a),
  offsets(offsets_a),
  flags(gc_flags_default)
{}

void destroy(void *p)
{
  if(p)
  {
    frame_t *fp = CLIENT_TO_FRAME(p);
    if(!fp->finalised)
      fp->collector->deallocate(fp);
   }
}

// b may be 0
void _init_ptr(void **a, void *b)
{
  *a = b;
}

// *a or b may be 0
void _set_ptr(void **a, void *b)
{
  *a = b;
}

// *a may be 0
void _release_ptr(void **a)
{
  *a = 0;
}

void _destroy_ptr(void **a)
{
  void *b = *a; // save the pointer value
  *a=0;         // null out the variable
  destroy(b);
}

/*
void reset_shape(void *memory, gc_shape_t &shape)
{
  assert(memory);
  CLIENT_TO_FRAME(memory)->shape = &shape;
}
*/

void set_used(void *memory, unsigned long n)
{
  assert(memory);
  assert(n>=0);
  assert(n<=get_count(memory));
  CLIENT_TO_FRAME(memory)->n_used = n;
}

void incr_used(void *memory, unsigned long n)
{
  set_used(memory, get_used(memory)+n);
}

unsigned long get_used(void *memory)
{
  assert(memory);
  return CLIENT_TO_FRAME(memory)->n_used;
}

unsigned long get_count(void *memory)
{
  assert(memory);
  return CLIENT_TO_FRAME(memory)->n_objects;
}

collector_t::collector_t() : debug(false) {}

void *create_empty_array(
  flx::gc::generic::collector_t &collector,
  flx::gc::generic::gc_shape_t &shape,
  unsigned long count
)
{
  void *p = collector.allocate(&shape,count);
  set_used (p, 0);
  return p;
}

}}} // end namespaces

// in global namespace now ..
void *operator new(
  std::size_t amt,
  flx::gc::generic::collector_t &collector,
  flx::gc::generic::gc_shape_t &shape
)
{
  if (amt != shape.amt)
  {
    fprintf(stderr,"Shape size error: allocator size = %ld\n",amt);
    fprintf(stderr,"Shape %s size = %ld\n",shape.cname,shape.amt);
    abort();
  }
  void *p = collector.allocate(&shape,1);
  flx::gc::generic::set_used (p, 1);
  return p;
}

